# coding: utf-8
# 

"""
SCL     D0
SDA in  D1
SDA out D2

https://ben-gillett.medium.com/i2c-and-more-with-the-ftdi-ft2232h-mini-module-and-an-arduino-ce0982578fed
"""
import time
import ctypes
from enum import Enum

class Channel():
    def __init__(self, name, index):
        self.name = name
        self.index = index
        self.handle = ctypes.c_void_p()

        
class ChannelConfig(ctypes.Structure):
    _fields_ = [('ClockRate', ctypes.c_int),('LatencyTimer', ctypes.c_ubyte),('Options', ctypes.c_int)]

    
class ChannelInfo(ctypes.Structure):
    _fields_ = [('Flags', ctypes.c_ulong),('Type', ctypes.c_ulong),('ID', ctypes.c_ulong),
            ('LocId', ctypes.c_ulong),('SerialNumber', ctypes.c_char*16),('Description', ctypes.c_char*64),
            ('ftHandle', ctypes.c_void_p)]

# Some constants for FTDI config...
START_BIT                   = 0x01
STOP_BIT                    = 0x02
BREAK_ON_NACK               = 0x04
NACK_LAST_BYTE              = 0x08
FAST_TRANSFER_BYTES         = 0x10
FAST_TRANSFER_BITS          = 0x20
FAST_TRANSFER               = 0x30
NO_ADDRESS                  = 0x40
I2C_DISABLE_3PHASE_CLOCKING = 0x01
I2C_ENABLE_DRIVE_ONLY_ZERO  = 0x02


class FT_STATUS(Enum):
    FT_OK                             = 0
    FT_INVALID_HANDLE                 = 1
    FT_DEVICE_NOT_FOUND               = 2
    FT_DEVICE_NOT_OPENED              = 3
    FT_IO_ERROR                       = 4
    FT_INSUFFICIENT_RESOURCES         = 5
    FT_INVALID_PARAMETER              = 6
    FT_INVALID_BAUD_RATE              = 7
    FT_DEVICE_NOT_OPENED_FOR_ERASE    = 8
    FT_DEVICE_NOT_OPENED_FOR_WRITE    = 9
    FT_FAILED_TO_WRITE_DEVICE         = 10
    FT_EEPROM_READ_FAILED             = 11
    FT_EEPROM_WRITE_FAILED            = 12
    FT_EEPROM_ERASE_FAILED            = 13
    FT_EEPROM_NOT_PRESENT             = 14
    FT_EEPROM_NOT_PROGRAMMED          = 15
    FT_INVALID_ARGS                   = 16
    FT_NOT_SUPPORTED                  = 17
    FT_OTHER_ERROR                    = 18
    FT_DEVICE_LIST_NOT_READ           = 19
    

status = lambda code: FT_STATUS(code).name

sleep_us = lambda x: time.sleep( x/1000000.0)
sleep_ms = lambda x: time.sleep( x/1000.0)
sleep = time.sleep

libMPSSE = ctypes.cdll.LoadLibrary("./libMPSSE.dll")
print('Listing channels...')
libMPSSE.Init_libMPSSE()
channel_count = ctypes.c_int()
ret = libMPSSE.I2C_GetNumChannels(ctypes.byref(channel_count))
print(f'Found {channel_count.value} channels (status {status(ret)})')
c = Channel('B', 0) # Open Channel B (index 0) for I2C

channel_info = ChannelInfo()
print(f'Getting info for channel with index {c.index}...')
ret = libMPSSE.I2C_GetChannelInfo(c.index, ctypes.byref(channel_info))
print(f'Channel description: {channel_info.Description.decode()} (status {status(ret)})')
ret = libMPSSE.I2C_OpenChannel(c.index, ctypes.byref(c.handle))
print(f'Channel {c.name} opened with handle: 0x{c.handle.value:x} (status {status(ret)})')
channel_conf = ChannelConfig(400000, 25, 0) # 400KHz, 25ms latency timer, no options
ret = libMPSSE.I2C_InitChannel(c.handle, ctypes.byref(channel_conf))
print(f'InitChannel() {c.name} (status {status(ret)})')

HTU21D_I2CADDR     = 0x40
HTU21D_READTEMP    = 0xE3
HTU21D_READHUM     = 0xE5
HTU21D_WRITEREG    = 0xE6
HTU21D_READREG     = 0xE7
HTU21D_RESET       = 0xFE

write_address = HTU21D_I2CADDR
mode = START_BIT | STOP_BIT | NACK_LAST_BYTE

buf = ctypes.c_buffer(b'', 1)
buf[0] = HTU21D_READTEMP
bytes_transfered = ctypes.c_int()
ret = libMPSSE.I2C_DeviceWrite(c.handle, write_address, len(buf), buf, ctypes.byref(bytes_transfered), mode)
print(f'Wrote {bytes_transfered.value} byte(s) (status {status(ret)})')
sleep_ms( 50 )
buf = ctypes.c_buffer(b'', 3)
bytes_transfered = ctypes.c_int()
ret = libMPSSE.I2C_DeviceRead(c.handle, write_address, len(buf), buf, ctypes.byref(bytes_transfered), mode)
print(f'Read {bytes_transfered.value} byte(s) (status {status(ret)})')

t = ( bytes(buf)[0] << 8) + ( bytes(buf)[1] & 0b11111100 )
temp = t * 175.72 / 65536 - 46.85
print( "*******************\nreaded temp: {0:.3f}\n*******************".format(  temp ) )

ret = libMPSSE.I2C_CloseChannel(c.handle)
print(f'CloseChannel() {c.name} (status {status(ret)})')




